Sulid

Struct Sulid 

pub struct Sulid { /* private fields */ }
A “signed” ULID, whose “random” portion is generated by a cryptographic signature.

Implementations

Source

impl Sulid

pub fn sign(secret: &SigningKey, payload: UnsignedPayload) -> SignedPayload

Create a new sULID by signing a payload.

Returns a SignedPayload which includes the generated Sulid, as well as the payload parts required to verify it.

The SignedPayload generated by this function is guaranteed to be valid. However, to validate untrusted sULIDs, you can construct a SignedPayload and call its is_valid().

pub fn timestamp(&self) -> SystemTime

Gets the timestamp portion of the sULID from the first 48 bits.

pub fn to_bytes(&self) -> [u8; 16]