pub struct SignedPayload {
pub sulid: Sulid,
pub public_key: VerifyingKey,
pub signature: Signature,
pub message_hash: Hash,
pub message_length: u64,
pub app_metadata: Vec<u8>,
}Expand description
Output of Sulid::sign, also used to verify sULIDs.
This contains the generated sULID and the necessary context to validate it.
The sULID generated by Sulid::sign will always be valid, so no need to revalidate it.
However, if you want to validate an untrusted sULID, construct this SignedPayload and check
SignedPayload::is_valid()
Fields
sulid: Sulidpublic_key: VerifyingKeysignature: Signaturemessage_hash: Hashmessage_length: u64app_metadata: Vec<u8>Implementations